@@ -20,7 +20,7 @@ from paste.util.mimeparse import desired_matches
 
 __all__ = ['WSGIRequest', 'WSGIResponse']
 
-_CHARSET_RE = re.compile(r'.*;\s*charset=(.*?)(;|$)', re.I)
+_CHARSET_RE = re.compile(r';\s*charset=([^;]*)', re.I)
 
 class DeprecatedSettings(StackedObjectProxy):
     def _push_object(self, obj):
@@ -99,14 +99,14 @@ class WSGIRequest(object):
     You are free to subclass this object.
 
     """
-    defaults = StackedObjectProxy(default=dict(charset=None, errors='strict',
+    defaults = StackedObjectProxy(default=dict(charset=None, errors='replace',
                                                decode_param_names=False,
                                                language='en-us'))
     def __init__(self, environ):
         self.environ = environ
         # This isn't "state" really, since the object is derivative:
         self.headers = EnvironHeaders(environ)
-
+        
         defaults = self.defaults._current_obj()
         self.charset = defaults.get('charset')
         if self.charset:
@@ -124,7 +124,19 @@ class WSGIRequest(object):
     method = environ_getter('REQUEST_METHOD')
     script_name = environ_getter('SCRIPT_NAME')
     path_info = environ_getter('PATH_INFO')
-    urlvars = environ_getter('paste.urlvars', default_factory=dict)
+
+    def urlvars(self):
+        """
+        Return any variables matched in the URL (e.g.,
+        ``wsgiorg.routing_args``).
+        """
+        if 'paste.urlvars' in self.environ:
+            return self.environ['paste.urlvars']
+        elif 'wsgiorg.routing_args' in self.environ:
+            return self.environ['wsgiorg.routing_args'][1]
+        else:
+            return {}
+    urlvars = property(urlvars, doc=urlvars.__doc__)
     
     def is_xhr(self):
         """Returns a boolean if X-Requested-With is present and a XMLHttpRequest"""
@@ -144,7 +156,7 @@ class WSGIRequest(object):
         if self._languages is not None:
             return self._languages
         acceptLanguage = self.environ.get('HTTP_ACCEPT_LANGUAGE')
-        langs = ACCEPT_LANGUAGE.parse(acceptLanguage)
+        langs = ACCEPT_LANGUAGE.parse(self.environ)
         fallback = self.defaults.get('language', 'en-us')
         if not fallback:
             return langs
@@ -245,7 +257,7 @@ class WSGIRequest(object):
         Determine the encoding as specified by the browser via the
         Content-Type's charset parameter, if one is set
         """
-        charset_match = _CHARSET_RE.match(self.headers.get('Content-Type', ''))
+        charset_match = _CHARSET_RE.search(self.headers.get('Content-Type', ''))
         if charset_match:
             return charset_match.group(1)
 
@@ -282,8 +294,8 @@ class WSGIResponse(object):
 
     """
     defaults = StackedObjectProxy(
-        default=dict(content_type='text/html',
-                     charset='UTF-8', errors='strict')
+        default=dict(content_type='text/html', charset='utf-8', 
+                     errors='strict', headers={'Cache-Control':'no-cache'})
         )
     def __init__(self, content='', mimetype=None, code=200):
         self._iter = None
@@ -300,6 +312,7 @@ class WSGIResponse(object):
             charset = defaults.get('charset')
             if charset:
                 mimetype = '%s; charset=%s' % (mimetype, charset)
+        self.headers.update(defaults.get('headers', {}))
         self.headers['Content-Type'] = mimetype
         self.errors = defaults.get('errors', 'strict')
 
@@ -351,7 +364,7 @@ class WSGIResponse(object):
         Determine the encoding as specified by the Content-Type's charset
         parameter, if one is set
         """
-        charset_match = _CHARSET_RE.match(self.headers.get('Content-Type', ''))
+        charset_match = _CHARSET_RE.search(self.headers.get('Content-Type', ''))
         if charset_match:
             return charset_match.group(1)
     
@@ -445,6 +458,80 @@ class WSGIResponse(object):
                 'is an iterator)' % self.__class__.__name__
         return sum([len(chunk) for chunk in self._iter])
 
+    ########################################
+    ## Content-type and charset
+
+    def charset__get(self):
+        """
+        Get/set the charset (in the Content-Type)
+        """
+        header = self.headers.get('content-type')
+        if not header:
+            return None
+        match = _CHARSET_RE.search(header)
+        if match:
+            return match.group(1)
+        return None
+
+    def charset__set(self, charset):
+        if charset is None:
+            del self.charset
+            return
+        try:
+            header = self.headers.pop('content-type')
+        except KeyError:
+            raise AttributeError(
+                "You cannot set the charset when no content-type is defined")
+        match = _CHARSET_RE.search(header)
+        if match:
+            header = header[:match.start()] + header[match.end():]
+        header += '; charset=%s' % charset
+        self.headers['content-type'] = header
+
+    def charset__del(self):
+        try:
+            header = self.headers.pop('content-type')
+        except KeyError:
+            # Don't need to remove anything
+            return
+        match = _CHARSET_RE.search(header)
+        if match:
+            header = header[:match.start()] + header[match.end():]
+        self.headers['content-type'] = header
+
+    charset = property(charset__get, charset__set, charset__del, doc=charset__get.__doc__)
+
+    def content_type__get(self):
+        """
+        Get/set the Content-Type header (or None), *without* the
+        charset or any parameters.
+
+        If you include parameters (or ``;`` at all) when setting the
+        content_type, any existing parameters will be deleted;
+        otherwise they will be preserved.
+        """
+        header = self.headers.get('content-type')
+        if not header:
+            return None
+        return header.split(';', 1)[0]
+
+    def content_type__set(self, value):
+        if ';' not in value:
+            header = self.headers.get('content-type', '')
+            if ';' in header:
+                params = header.split(';', 1)[1]
+                value += ';' + params
+        self.headers['content-type'] = value
+
+    def content_type__del(self):
+        try:
+            del self.headers['content-type']
+        except KeyError:
+            pass
+
+    content_type = property(content_type__get, content_type__set,
+                            content_type__del, doc=content_type__get.__doc__)
+
 ## @@ I'd love to remove this, but paste.httpexceptions.get_exception
 ##    doesn't seem to work...
 # See http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
@@ -458,6 +545,7 @@ STATUS_CODE_TEXT = {
     204: 'NO CONTENT',
     205: 'RESET CONTENT',
     206: 'PARTIAL CONTENT',
+    226: 'IM USED',
     300: 'MULTIPLE CHOICES',
     301: 'MOVED PERMANENTLY',
     302: 'FOUND',
